package models

import (
	"errors"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"hongze/hongze_edb_lib/utils"
	"strconv"
	"strings"
	"time"
)

// AddCalculateTbz 同比值
func AddCalculateTbz(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName string) (edbInfo *EdbInfo, err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("AddCalculateTbz,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	if req.EdbInfoId <= 0 {
		edbInfo = new(EdbInfo)
		edbInfo.Source = utils.DATA_SOURCE_CALCULATE_TBZ
		edbInfo.SourceName = "同比值"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		newEdbInfoId, tmpErr := to.Insert(edbInfo)
		if tmpErr != nil {
			err = tmpErr
			return
		}
		edbInfo.EdbInfoId = int(newEdbInfoId)

		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			_, err = to.Insert(calculateMappingItem)
			if err != nil {
				return
			}
		}

	} else {
		edbInfo, err = GetEdbInfoById(req.EdbInfoId)
		if err != nil {
			return
		}
		dataTableName := GetEdbDataTableName(utils.DATA_SOURCE_CALCULATE_TBZ)
		deleteSql := ` DELETE FROM %s WHERE edb_info_id=? `
		deleteSql = fmt.Sprintf(deleteSql, dataTableName)
		_, err = to.Raw(deleteSql, req.EdbInfoId).Exec()
		if err != nil {
			return
		}
	}

	//计算数据
	err = refreshAllCalculateTbz(to, edbInfo.EdbInfoId, edbInfo.Source, fromEdbInfo, edbCode, "", "")

	return
}

// EditCalculateTbz 同比值
func EditCalculateTbz(edbInfo *EdbInfo, req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo) (err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("EditCalculateTbz,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//修改指标信息
	edbInfo.EdbName = req.EdbName
	edbInfo.EdbNameSource = req.EdbName
	edbInfo.Frequency = req.Frequency
	edbInfo.Unit = req.Unit
	edbInfo.ClassifyId = req.ClassifyId
	edbInfo.ModifyTime = time.Now()
	_, err = to.Update(edbInfo, "EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "ModifyTime")
	if err != nil {
		return
	}

	//判断计算指标是否被更换
	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
	existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

	count, err := GetEdbInfoCalculateCountByCondition(existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count > 0 { // 指标未被替换，无需重新计算
		return
	}

	//删除，计算指标关联的，基础指标的关联关系
	sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
	_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
	if err != nil {
		return
	}
	//清空原有数据
	sql = ` DELETE FROM edb_data_calculate_tbz WHERE edb_info_id = ? `
	_, err = to.Raw(sql, edbInfo.EdbInfoId).Exec()
	if err != nil {
		return
	}
	//关联关系
	{
		calculateMappingItem := &EdbInfoCalculateMapping{
			EdbInfoCalculateMappingId: 0,
			EdbInfoId:                 edbInfo.EdbInfoId,
			Source:                    utils.DATA_SOURCE_CALCULATE_TBZ,
			SourceName:                "同比值",
			EdbCode:                   edbInfo.EdbCode,
			FromEdbInfoId:             fromEdbInfo.EdbInfoId,
			FromEdbCode:               fromEdbInfo.EdbCode,
			FromEdbName:               fromEdbInfo.EdbName,
			FromSource:                fromEdbInfo.Source,
			FromSourceName:            fromEdbInfo.SourceName,
			FromTag:                   "",
			Sort:                      1,
			CreateTime:                time.Now(),
			ModifyTime:                time.Now(),
		}
		_, err = to.Insert(calculateMappingItem)
		if err != nil {
			return
		}
	}

	//计算数据
	err = refreshAllCalculateTbz(to, edbInfo.EdbInfoId, edbInfo.Source, fromEdbInfo, edbInfo.EdbCode, "", "")

	return
}

// RefreshAllCalculateTbz 刷新所有同比值数据
func RefreshAllCalculateTbz(edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	o := orm.NewOrm()
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			fmt.Println("RefreshAllCalculateTbz,Err:" + err.Error())
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//计算数据
	err = refreshAllCalculateTbz(to, edbInfoId, source, fromEdbInfo, edbCode, startDate, endDate)
	return
}

// refreshAllCalculateTbz
func refreshAllCalculateTbz(to orm.TxOrmer, edbInfoId, source int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string) (err error) {
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, fromEdbInfo.EdbInfoId)

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}
	//if endDate != "" {
	//	condition += " AND data_time<=? "
	//	pars = append(pars, endDate)
	//}

	dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}
	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
	}
	fmt.Println("source:", source)
	//获取指标所有数据
	existDataList := make([]*EdbData, 0)
	dataTableName := GetEdbDataTableName(source)
	fmt.Println("dataTableName:", dataTableName)
	sql := `SELECT * FROM %s WHERE edb_info_id=? `
	sql = fmt.Sprintf(sql, dataTableName)
	_, err = to.Raw(sql, edbInfoId).QueryRows(&existDataList)
	if err != nil {
		return err
	}
	existDataMap := make(map[string]string)
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v.Value
	}
	fmt.Println("existDataMap:", existDataMap)
	addSql := ` INSERT INTO edb_data_calculate_tbz(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	existAddDataMap := make(map[string]string)
	for _, av := range dateArr {
		currentItem := dataMap[av]
		if currentItem != nil {
			//当前日期
			currentDate, err := time.ParseInLocation(utils.FormatDate, av, time.Local)
			if err != nil {
				return err
			}
			//上一年的日期
			preDate := currentDate.AddDate(-1, 0, 0)
			preDateStr := preDate.Format(utils.FormatDate)
			if findItem, ok := dataMap[preDateStr]; ok { //上一年同期找到
				//dataTime, _ := time.ParseInLocation(utils.FormatDate, date, time.Local)
				timestamp := currentDate.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				val := TbzDiv(currentItem.Value, findItem.Value)

				if existVal, ok := existDataMap[av]; !ok {
					if _, existOk := existAddDataMap[av]; !existOk {
						addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
						isAdd = true
					}
					existAddDataMap[av] = av
				} else {
					existValDecimal, err := decimal.NewFromString(existVal)
					existStr := existValDecimal.String()
					if existStr != val {
						sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
						sql = fmt.Sprintf(sql, dataTableName)
						_, err = to.Raw(sql, val, edbInfoId, av).Exec()
						if err != nil {
							return err
						}
					}
				}
				continue
			} else {
				if fromEdbInfo.Frequency == "月度" { //向上和向下，各找一个月
					for i := 0; i <= 35; i++ {
						nextDateDay := preDate.AddDate(0, 0, i)
						nextDateDayStr := nextDateDay.Format(utils.FormatDate)
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TbzDiv(currentItem.Value, findItem.Value)

							if existVal, ok := existDataMap[av]; !ok {
								if _, existOk := existAddDataMap[av]; !existOk {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existAddDataMap[av] = av
							} else {
								existValDecimal, err := decimal.NewFromString(existVal)
								existStr := existValDecimal.String()
								if existStr != val {
									sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
									sql = fmt.Sprintf(sql, dataTableName)
									_, err = to.Raw(sql, val, edbInfoId, av).Exec()
									if err != nil {
										return err
									}
								}
							}
							break
						} else {
							preDateDay := preDate.AddDate(0, 0, -i)
							preDateDayStr := preDateDay.Format(utils.FormatDate)
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TbzDiv(currentItem.Value, findItem.Value)

								if existVal, ok := existDataMap[av]; !ok {
									if _, existOk := existAddDataMap[av]; !existOk {
										addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
										isAdd = true
									}
									existAddDataMap[av] = av
								} else {
									existValDecimal, err := decimal.NewFromString(existVal)
									existStr := existValDecimal.String()
									if existStr != val {
										sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
										sql = fmt.Sprintf(sql, dataTableName)
										_, err = to.Raw(sql, val, edbInfoId, av).Exec()
										if err != nil {
											return err
										}
									}
								}
								break
							}
						}
					}
				} else if fromEdbInfo.Frequency == "季度" || fromEdbInfo.Frequency == "年度" {
					if findItem, ok := dataMap[preDateStr]; ok { //上一年同期->下一个月找到
						timestamp := currentDate.UnixNano() / 1e6
						timestampStr := fmt.Sprintf("%d", timestamp)
						val := TbzDiv(currentItem.Value, findItem.Value)

						if existVal, ok := existDataMap[av]; !ok {
							if _, existOk := existAddDataMap[av]; !existOk {
								addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
								isAdd = true
							}
							existAddDataMap[av] = av
						} else {
							existValDecimal, err := decimal.NewFromString(existVal)
							existStr := existValDecimal.String()
							if existStr != val {
								sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
								sql = fmt.Sprintf(sql, dataTableName)
								_, err = to.Raw(sql, val, edbInfoId, av).Exec()
								if err != nil {
									return err
								}
							}
						}
						break
					}
				} else {
					nextDateDay := preDate.AddDate(0, 0, 1)
					nextDateDayStr := nextDateDay.Format(utils.FormatDate)

					preDateDay := preDate.AddDate(0, 0, -1)
					preDateDayStr := preDateDay.Format(utils.FormatDate)

					for i := 0; i < 35; i++ {
						if i >= 1 {
							nextDateDay = nextDateDay.AddDate(0, 0, i)
							nextDateDayStr = nextDateDay.Format(utils.FormatDate)
						}
						if findItem, ok := dataMap[nextDateDayStr]; ok { //上一年同期->下一个月找到
							timestamp := currentDate.UnixNano() / 1e6
							timestampStr := fmt.Sprintf("%d", timestamp)
							val := TbzDiv(currentItem.Value, findItem.Value)

							if existVal, ok := existDataMap[av]; !ok {
								if _, existOk := existAddDataMap[av]; !existOk {
									addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
									isAdd = true
								}
								existAddDataMap[av] = av
							} else {
								existValDecimal, err := decimal.NewFromString(existVal)
								existStr := existValDecimal.String()
								if existStr != val {
									sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
									sql = fmt.Sprintf(sql, dataTableName)
									_, err = to.Raw(sql, val, edbInfoId, av).Exec()
									if err != nil {
										return err
									}
								}
							}
							break
						} else {
							if i >= 1 {
								preDateDay = preDateDay.AddDate(0, 0, -i)
								preDateDayStr = preDateDay.Format(utils.FormatDate)
							}
							if findItem, ok := dataMap[preDateDayStr]; ok { //上一年同期->上一个月找到
								timestamp := currentDate.UnixNano() / 1e6
								timestampStr := fmt.Sprintf("%d", timestamp)
								val := TbzDiv(currentItem.Value, findItem.Value)

								if existVal, ok := existDataMap[av]; !ok {
									if _, existOk := existAddDataMap[av]; !existOk {
										addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, val)
										isAdd = true
									}
									existAddDataMap[av] = av
								} else {
									existValDecimal, err := decimal.NewFromString(existVal)
									existStr := existValDecimal.String()
									if existStr != val {
										sql := ` UPDATE %s SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
										sql = fmt.Sprintf(sql, dataTableName)
										_, err = to.Raw(sql, val, edbInfoId, av).Exec()
										if err != nil {
											return err
										}
									}
								}
								break
							}
						}
					}
				}
			}
		}
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = to.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

func TbzDiv(a, b float64) string {
	var valStr string
	if b != 0 {
		af := decimal.NewFromFloat(a)
		bf := decimal.NewFromFloat(b)
		val, _ := af.Div(bf).Float64()
		val = val - 1
		valStr = decimal.NewFromFloat(val).RoundCeil(4).String()
	} else {
		valStr = "0"
	}
	return valStr
}
